/**
***     GUIApp.c
***
***     ̃\[XR[h́AG[XgbvTXyhȂǂ̏Ă܂B
***     Copyright (C) 2006 SophiaCradle Incorporated.
***     All rights reserved.
**/

#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEStdLib.h"
#include "AEEShell.h"
#include "GUIApp.bid"
#include "AEEGraphics.h"
#include "AEEText.h"

#define     MARGIN                      (6)
#define     CHECKBOX_SIZE               (16)
#define     CHECKBOX_POINT1_X           (3)
#define     CHECKBOX_POINT1_Y           (8)
#define     CHECKBOX_POINT2_X           (6)
#define     CHECKBOX_POINT2_Y           (11)
#define     CHECKBOX_POINT3_X           (12)
#define     CHECKBOX_POINT3_Y           (4)
#define     RADIOBUTTON_CIRCLE_SIZE     (12)

// R|[lgŎgp\́iR{{bNXj
typedef struct ComboBox {
    int selected;
    int size;
    AECHAR** texts;
    int index;
    int cursor;
    AEERect inner_rect;
    AEERect selected_rect;
} ComboBox;

// R|[lgŎgp\́iWI{^j
typedef struct RadioButton {
    int checked;
    AEERect text_rect;
    AEECircle circle;
    AEECircle inner_circle;
} RadioButton;

// R|[lgŎgp\́i`FbN{bNXj
typedef struct CheckBox {
    int checked;
    AEERect inner_rect;
    AEERect string_rect;
    AEELine line1;
    AEELine line2;
} CheckBox;

// R|[lg̎
typedef enum ComponentType {
    COMPONENT_NULL = -1,
    COMPONENT_LABEL,
    COMPONENT_TEXTFIELD,
    COMPONENT_COMBOBOX,
    COMPONENT_RADIOBUTTON,
    COMPONENT_CHECKBOX,
    COMPONENT_BUTTON
} ComponentType;

// R|[lg
typedef struct Component {
    // 
    ComponentType type;

    // ̈
    AEERect rect;

    // eLXg
    AECHAR* text;

    // R|[lg̎ނɈˑϐ
    union {
        ITextCtl* textctl; // TEXTFIELD
        CheckBox checkbox;
        RadioButton radiobutton;
        ComboBox combobox;
    } component;
} Component;

// Avbg\
typedef struct GUIApp {
    AEEApplet a; // Avbg\̂̐擪o͕K AEEApplet ^ɂ邱ƁB

    // ʕ`p
    IGraphics* graphics;
    int width;
    int height;
    int font_height;

    // R|[lg
    Component components[20];
    int component_length; // R|[lg̐
    int component_index; // ݃tH[JXR|[lg̃CfbNX
} GUIApp;

static boolean GUIApp_HandleEvent(GUIApp* app, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static void Draw(GUIApp* app);
static boolean OnAppStart(GUIApp* app);
static boolean OnAppStop(GUIApp* app);
static boolean OnKey(GUIApp* app, uint16 key);
static void SetFocusNextComponent(GUIApp* app);
static void SetFocusPreviousComponent(GUIApp* app);

// R|[lg̊֐
static void Component_Init(Component* component, GUIApp* app, ComponentType type, AEERect* rect, char* text);
static void Component_SetTextFieldSize(Component* component, uint16 size);
static void Component_SetComboBoxSize(Component* component, GUIApp* app, int size);
static void Component_AddElementToComboBox(Component* component, char* text);
static void Component_Free(Component* component);
static void Component_Draw(Component* component, GUIApp* app, int isFocused);
static int Component_SetFocused(Component* component);
static void Component_Select(Component* component, GUIApp* app);
static int Component_HandleEvent(Component* component, GUIApp* app, AEEEvent evt, uint16 wp, uint32 dwp);

int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
    *ppObj = NULL;
    if (ClsId == AEECLSID_GUIAPP) {
        if (AEEApplet_New(sizeof(GUIApp), ClsId, pIShell, po, (IApplet**)ppObj, (AEEHANDLER)GUIApp_HandleEvent, NULL) == TRUE) {
            return AEE_SUCCESS;
        }
    }
    return EFAILED;
}

static boolean GUIApp_HandleEvent(GUIApp* app, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
    if (eCode == EVT_APP_START) {
        return OnAppStart(app);
    }
    else if (Component_HandleEvent(app->components + app->component_index, app, eCode, wParam, dwParam)) {
        // R|[lgCxgꍇ
        Draw(app);
        return TRUE;
    }
    else {
        switch (eCode) {
            case EVT_APP_STOP:
                return OnAppStop(app);

            case EVT_KEY:
                return OnKey(app, wParam);
        }
    }
    return FALSE;
}

// ʕ`
static void Draw(GUIApp* app)
{
    int i;
    AEERect rect;

    rect.x = 0;
    rect.y = 0;
    rect.dx = app->width;
    rect.dy = app->height;
    // ʂNA
    IDISPLAY_FillRect(app->a.m_pIDisplay, &rect, RGB_WHITE);
    // Ō̃R|[lg`
    for (i = app->component_length - 1; i >= 0; i--) {
        Component_Draw(&app->components[i], app, i == app->component_index);
    }
    IDISPLAY_Update(app->a.m_pIDisplay);
    IGRAPHICS_Update(app->graphics);
    return;
}

// AvbgJnƂɌĂяoB
static boolean OnAppStart(GUIApp* app)
{
    AEEDeviceInfo devinfo;
    AEERect r;
    IShell* shell = app->a.m_pIShell;
    AECHAR wstring[16];

    // ʃTCY擾
    ISHELL_GetDeviceInfo(shell, &devinfo);
    app->width = devinfo.cxScreen;
    app->height = devinfo.cyScreen;
    // tHg擾
    app->font_height = IDISPLAY_GetFontMetrics(app->a.m_pIDisplay, AEE_FONT_NORMAL, NULL, NULL);

    app->graphics = NULL;
    ISHELL_CreateInstance(shell, AEECLSID_GRAPHICS, (void**)&app->graphics);

    app->component_length = 0;
    app->component_index = 0;
    // R|[lgǉ
    // x
    r.x = 0;
    r.y = 0;
    r.dx = app->width;
    r.dy = app->font_height;
    Component_Init(&app->components[app->component_length], app, COMPONENT_LABEL, &r, "GUIApp");
    app->component_length++;

    // eLXgtB[h
    r.x = 0;
    r.y = r.dy + MARGIN;
    STRTOWSTR("2006", wstring, 16 * sizeof(AECHAR));
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    r.dy = app->font_height + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_TEXTFIELD, &r, "2006");
    Component_SetTextFieldSize(&app->components[app->component_length], 4);
    app->component_length++;

    r.x = r.x + r.dx + MARGIN;
    STRTOWSTR("N", wstring, 16 * sizeof(AECHAR));
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_LABEL, &r, "N");
    app->component_length++;

    r.x = r.x + r.dx + MARGIN;
    STRTOWSTR("10", wstring, 16 * sizeof(AECHAR));
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_TEXTFIELD, &r, "4");
    Component_SetTextFieldSize(&app->components[app->component_length], 2);
    app->component_length++;

    r.x = r.x + r.dx + MARGIN;
    STRTOWSTR("", wstring, 16 * sizeof(AECHAR));
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_LABEL, &r, "");
    app->component_length++;

    r.x = r.x + r.dx + MARGIN;
    STRTOWSTR("21", wstring, 16 * sizeof(AECHAR));
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_TEXTFIELD, &r, "21");
    Component_SetTextFieldSize(&app->components[app->component_length], 2);
    app->component_length++;

    r.x = r.x + r.dx + MARGIN;
    STRTOWSTR("", wstring, 16 * sizeof(AECHAR));
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_LABEL, &r, "");
    app->component_length++;

    r.x = 0;
    r.y = r.y + r.dy + MARGIN;
#if defined __arm
    STRTOWSTR("\\", wstring, 16 * sizeof(AECHAR));
#else
    STRTOWSTR("\", wstring, 16 * sizeof(AECHAR));
#endif
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    r.dy = app->font_height + MARGIN;
#if defined __arm
    Component_Init(&app->components[app->component_length], app, COMPONENT_LABEL, &r, "\\");
#else
    Component_Init(&app->components[app->component_length], app, COMPONENT_LABEL, &r, "\");
#endif
    app->component_length++;

    // R{{bNX
    r.x = r.x + r.dx + MARGIN;
    r.dx = app->width - r.x - MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_COMBOBOX, &r, NULL);
    Component_SetComboBoxSize(&app->components[app->component_length], app, 3);
    Component_AddElementToComboBox(&app->components[app->component_length], "c");
    Component_AddElementToComboBox(&app->components[app->component_length], "o");
    Component_AddElementToComboBox(&app->components[app->component_length], "̑");
    app->component_length++;

    r.x = 0;
    r.y = r.y + r.dy + MARGIN;
    STRTOWSTR("ڍ", wstring, 16 * sizeof(AECHAR));
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    r.dy = app->font_height + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_LABEL, &r, "ڍ");
    app->component_length++;

    r.x = r.x + r.dx + MARGIN;
    r.dx = app->width - r.x - MARGIN;
    r.dy = app->font_height * 2 + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_TEXTFIELD, &r, "");
    app->component_length++;

    r.x = 0;
    r.y = r.y + r.dy + MARGIN;
    STRTOWSTR("dvx", wstring, 16 * sizeof(AECHAR));
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    r.dy = app->font_height + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_LABEL, &r, "dvx");
    app->component_length++;

    // WI{^
    r.x = r.x + r.dx + MARGIN;
    STRTOWSTR("", wstring, 16 * sizeof(AECHAR));
    r.dx = RADIOBUTTON_CIRCLE_SIZE + IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_RADIOBUTTON, &r, "");
    app->component_length++;

    r.x = r.x + r.dx + MARGIN;
    STRTOWSTR("", wstring, 16 * sizeof(AECHAR));
    r.dx = RADIOBUTTON_CIRCLE_SIZE + IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_RADIOBUTTON, &r, "");
    app->component_length++;

    r.x = r.x + r.dx + MARGIN;
    STRTOWSTR("", wstring, 16 * sizeof(AECHAR));
    r.dx = RADIOBUTTON_CIRCLE_SIZE + IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_RADIOBUTTON, &r, "");
    app->component_length++;

    r.x = 0;
    r.y = r.y + r.dy + MARGIN;
    STRTOWSTR("A[", wstring, 16 * sizeof(AECHAR));
    r.dx = CHECKBOX_SIZE + IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    r.dy = app->font_height + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_CHECKBOX, &r, "A[");
    app->component_length++;

    // {^
    r.x = 0;
    r.y = r.y + r.dy + MARGIN;
    STRTOWSTR("쐬", wstring, 16 * sizeof(AECHAR));
    r.dx = IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    r.dy = app->font_height + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_BUTTON, &r, "쐬");
    app->component_length++;

    r.x = r.x + r.dx + MARGIN;
    STRTOWSTR("LZ", wstring, 16 * sizeof(AECHAR));
    r.dx = RADIOBUTTON_CIRCLE_SIZE + IDISPLAY_MeasureText(app->a.m_pIDisplay, AEE_FONT_NORMAL, wstring) + MARGIN;
    Component_Init(&app->components[app->component_length], app, COMPONENT_BUTTON, &r, "LZ");
    app->component_length++;

    Draw(app);
    return TRUE;
}

// AvbgIƂɌĂяoB
static boolean OnAppStop(GUIApp* app)
{
    int i;

    // Ō̃R|[lg
    for (i = app->component_length - 1; i >= 0; i--) {
        Component_Free(&app->components[i]);
    }
    IGRAPHICS_Release(app->graphics);
    return TRUE;
}

// L[ꂽƂɌĂяoB
static boolean OnKey(GUIApp* app, uint16 key)
{
    switch (key) {
        case AVK_DOWN:
        case AVK_RIGHT:
            SetFocusNextComponent(app);
            Draw(app);
            return TRUE;

        case AVK_UP:
        case AVK_LEFT:
            SetFocusPreviousComponent(app);
            Draw(app);
            return TRUE;

        case AVK_SELECT:
            // R|[lgZNg
            Component_Select(app->components + app->component_index, app);
            Draw(app);
            return TRUE;
    }
    return FALSE;
}

// ̃R|[lg
static void SetFocusNextComponent(GUIApp* app)
{
    int original = app->component_index;
    int index;

    // R|[lgz̏ԂɃ`FbN
	for (index = (original == app->component_length - 1) ? (0) : (original + 1); index != original; index = (index == app->component_length - 1) ? (0) : (index + 1)) {
        if (Component_SetFocused(app->components + index)) {
            app->component_index = index;
            break;
        }
    }
    return;
}

// ÕR|[lg
static void SetFocusPreviousComponent(GUIApp* app)
{
    int original = app->component_index;
    int index;

    // R|[lgz̋tɃ`FbN
    for (index = (original == 0) ? (app->component_length - 1) : (original - 1); index != original; index = (index == 0) ? (app->component_length - 1) : (index - 1)) {
        if (Component_SetFocused(app->components + index)) {
            app->component_index = index;
            break;
        }
    }
    return;
}

// R|[lg̏
static void Component_Init(Component* component, GUIApp* app, ComponentType type, AEERect* rect, char* text)
{
    component->type = type;
    component->rect = *rect;
    if (text != NULL) {
        int len = STRLEN(text) + 1;

        component->text = MALLOC(len * sizeof(AECHAR));
        STRTOWSTR(text, component->text, len * sizeof(AECHAR));
    }
    else {
        component->text = NULL;
    }
    switch (type) {
        case COMPONENT_LABEL:
            break;

        case COMPONENT_TEXTFIELD:
            component->component.textctl = NULL;
            ISHELL_CreateInstance(app->a.m_pIShell, AEECLSID_TEXTCTL, (void**)&component->component.textctl);
            ITEXTCTL_SetRect(component->component.textctl, &component->rect);
            ITEXTCTL_SetText(component->component.textctl, component->text, -1);
            ITEXTCTL_SetProperties(component->component.textctl, TP_FRAME | TP_FIXSETRECT | TP_MULTILINE);
            break;

        case COMPONENT_COMBOBOX:
            component->component.combobox.selected = FALSE;
            component->component.combobox.inner_rect.x = component->rect.x + 2;
            component->component.combobox.inner_rect.y = component->rect.y + 2;
            component->component.combobox.inner_rect.dx = component->rect.dx - 4;
            component->component.combobox.inner_rect.dy = component->rect.dy - 4;
            component->component.combobox.index = 0;
            component->component.combobox.cursor = 0;
            component->component.combobox.size = 0;
            break;

        case COMPONENT_RADIOBUTTON:
            component->component.radiobutton.checked = FALSE;
            component->component.radiobutton.circle.cx = rect->x + RADIOBUTTON_CIRCLE_SIZE / 2;
            component->component.radiobutton.circle.cy = rect->y + rect->dy / 2;
            component->component.radiobutton.circle.r = RADIOBUTTON_CIRCLE_SIZE / 2;
            component->component.radiobutton.inner_circle.cx = rect->x + RADIOBUTTON_CIRCLE_SIZE / 2;
            component->component.radiobutton.inner_circle.cy = rect->y + rect->dy / 2;
            component->component.radiobutton.inner_circle.r = RADIOBUTTON_CIRCLE_SIZE / 2 - 2;
            component->component.radiobutton.text_rect = component->rect;
            component->component.radiobutton.text_rect.x += RADIOBUTTON_CIRCLE_SIZE;
            break;

        case COMPONENT_CHECKBOX:
            component->component.checkbox.checked = FALSE;
            component->component.checkbox.inner_rect.x = rect->x + 1;
            component->component.checkbox.inner_rect.y = (rect->dy - CHECKBOX_SIZE) / 2 + rect->y;
            component->component.checkbox.inner_rect.dx = CHECKBOX_SIZE;
            component->component.checkbox.inner_rect.dy = CHECKBOX_SIZE;
            component->component.checkbox.line1.sx = component->component.checkbox.inner_rect.x + CHECKBOX_POINT1_X;
            component->component.checkbox.line1.sy = component->component.checkbox.inner_rect.y + CHECKBOX_POINT1_Y;
            component->component.checkbox.line1.ex = component->component.checkbox.inner_rect.x + CHECKBOX_POINT2_X;
            component->component.checkbox.line1.ey = component->component.checkbox.inner_rect.y + CHECKBOX_POINT2_Y;
            component->component.checkbox.line2.sx = component->component.checkbox.inner_rect.x + CHECKBOX_POINT2_X;
            component->component.checkbox.line2.sy = component->component.checkbox.inner_rect.y + CHECKBOX_POINT2_Y;
            component->component.checkbox.line2.ex = component->component.checkbox.inner_rect.x + CHECKBOX_POINT3_X;
            component->component.checkbox.line2.ey = component->component.checkbox.inner_rect.y + CHECKBOX_POINT3_Y;
            component->component.checkbox.string_rect = component->rect;
            component->component.checkbox.string_rect.x += CHECKBOX_SIZE + 2;
            break;

        case COMPONENT_BUTTON:
            break;
    }
    return;
}

// eLXgtB[hɕw
static void Component_SetTextFieldSize(Component* component, uint16 size)
{
    switch (component->type) {
        case COMPONENT_TEXTFIELD:
            ITEXTCTL_SetMaxSize(component->component.textctl, size);
            break;
    }
    return;
}

// R{{bNX̃TCYiI̐jw
static void Component_SetComboBoxSize(Component* component, GUIApp* app, int size)
{
    int i;

    switch (component->type) {
        case COMPONENT_COMBOBOX:
            component->component.combobox.size = size;
            component->component.combobox.texts = MALLOC(size * sizeof(AECHAR*));
            component->component.combobox.selected_rect.x = component->rect.x;
            component->component.combobox.selected_rect.y = component->rect.y;
            component->component.combobox.selected_rect.dx = component->rect.dx;
            component->component.combobox.selected_rect.dy = (app->font_height + 1) * size + 5;
            for (i = 0; i < size; i++) {
                component->component.combobox.texts[i] = NULL;
            }
            break;
    }
    return;
}

// R{{bNX̑Iǉ
static void Component_AddElementToComboBox(Component* component, char* text)
{
    int i;

    switch (component->type) {
        case COMPONENT_COMBOBOX:
            for (i = 0; i < component->component.combobox.size; i++) {
                if (component->component.combobox.texts[i] == NULL) {
                    int len = STRLEN(text) + 1;

                    component->component.combobox.texts[i] = MALLOC(len * sizeof(AECHAR));
                    STRTOWSTR(text, component->component.combobox.texts[i], len * sizeof(AECHAR));
                    break;
                }
            }
            break;
    }
    return;
}

// R|[lg̉
static void Component_Free(Component* component)
{
    int i;

    switch (component->type) {
        case COMPONENT_COMBOBOX:
            for (i = component->component.combobox.size - 1; i >= 0; i--) {
                if (component->component.combobox.texts[i] != NULL) {
                    FREE(component->component.combobox.texts[i]);
                }
            }
            FREE(component->component.combobox.texts);
            break;

        case COMPONENT_TEXTFIELD:
            ITEXTCTL_Release(component->component.textctl);
            break;
    }
    if (component->text != NULL) {
        FREE(component->text);
    }
    return;
}

// R|[lg̕`
static void Component_Draw(Component* component, GUIApp* app, int isFocused)
{
    switch (component->type) {
        case COMPONENT_LABEL:
            IDISPLAY_DrawText(app->a.m_pIDisplay, AEE_FONT_NORMAL, component->text, -1, 0, 0, &component->rect, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
            break;

        case COMPONENT_TEXTFIELD:
            ITEXTCTL_Redraw(component->component.textctl);
            IDISPLAY_DrawRect(app->a.m_pIDisplay, &component->rect, (isFocused) ? (MAKE_RGB(0, 255, 0)) : (MAKE_RGB(0, 0, 0)), 0, IDF_RECT_FRAME);
            break;

        case COMPONENT_COMBOBOX:
            if (component->component.combobox.selected) {
                AEERect inner = component->component.combobox.inner_rect;
                int i;

                IDISPLAY_DrawRect(app->a.m_pIDisplay, &component->component.combobox.selected_rect, MAKE_RGB(0, 0, 0), MAKE_RGB(255, 255, 255), IDF_RECT_FRAME | IDF_RECT_FILL);

                for (i = 0; i < component->component.combobox.size; i++) {
                    if (component->component.combobox.cursor == i) {
                        IDISPLAY_DrawRect(app->a.m_pIDisplay, &inner, MAKE_RGB(0, 255, 0), 0, IDF_RECT_FRAME);
                    }
                    IDISPLAY_DrawText(app->a.m_pIDisplay, AEE_FONT_NORMAL, component->component.combobox.texts[i], -1, 0, 0, &inner, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
                    inner.y += app->font_height + 1;
                }
            }
            else {
                IDISPLAY_DrawRect(app->a.m_pIDisplay, &component->rect, (isFocused) ? (MAKE_RGB(0, 255, 0)) : (MAKE_RGB(0, 0, 0)), 0, IDF_RECT_FRAME);
                IDISPLAY_DrawText(app->a.m_pIDisplay, AEE_FONT_NORMAL, component->component.combobox.texts[component->component.combobox.index], -1, 0, 0, &component->component.combobox.inner_rect, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
            }
            break;

        case COMPONENT_RADIOBUTTON:
            if (isFocused) {
                IGRAPHICS_SetColor(app->graphics, 0, 255, 0, 0);
            }
            else {
                IGRAPHICS_SetColor(app->graphics, 0, 0, 0, 0);
            }
            IGRAPHICS_SetFillMode(app->graphics, FALSE);
            IGRAPHICS_DrawCircle(app->graphics, &component->component.radiobutton.circle);

            if (component->component.radiobutton.checked) {
                IGRAPHICS_SetColor(app->graphics, 0, 0, 0, 0);
                IGRAPHICS_SetFillColor(app->graphics, 0, 0, 0, 0);
                IGRAPHICS_SetFillMode(app->graphics, TRUE);
                IGRAPHICS_DrawCircle(app->graphics, &component->component.radiobutton.inner_circle);
            }
            IDISPLAY_DrawText(app->a.m_pIDisplay, AEE_FONT_NORMAL, component->text, -1, 0, 0, &component->component.radiobutton.text_rect, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
            break;

        case COMPONENT_CHECKBOX:
            if (isFocused) {
                IGRAPHICS_SetColor(app->graphics, 0, 255, 0, 0);
            }
            else {
                IGRAPHICS_SetColor(app->graphics, 0, 0, 0, 0);
            }
            IGRAPHICS_SetFillMode(app->graphics, FALSE);
            IGRAPHICS_DrawRect(app->graphics, &component->component.checkbox.inner_rect);

            if (component->component.checkbox.checked) {
                IGRAPHICS_SetColor(app->graphics, 0, 0, 0, 0);
                IGRAPHICS_DrawLine(app->graphics, &component->component.checkbox.line1);
                IGRAPHICS_DrawLine(app->graphics, &component->component.checkbox.line2);
            }
            IDISPLAY_DrawText(app->a.m_pIDisplay, AEE_FONT_NORMAL, component->text, -1, 0, 0, &component->component.checkbox.string_rect, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
            break;

        case COMPONENT_BUTTON:
            IDISPLAY_DrawRect(app->a.m_pIDisplay, &component->rect, (isFocused) ? (MAKE_RGB(0, 255, 0)) : (MAKE_RGB(0, 0, 0)), 0, IDF_RECT_FRAME);
            IDISPLAY_DrawText(app->a.m_pIDisplay, AEE_FONT_NORMAL, component->text, -1, 0, 0, &component->rect, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
            break;
    }
    return;
}

// R|[lgtH[JX擾
static int Component_SetFocused(Component* component)
{
    switch (component->type) {
        case COMPONENT_LABEL:
            return FALSE;

        case COMPONENT_TEXTFIELD:
            ITEXTCTL_SetActive(component->component.textctl, TRUE);
            return TRUE;

        case COMPONENT_COMBOBOX:
        case COMPONENT_RADIOBUTTON:
        case COMPONENT_CHECKBOX:
        case COMPONENT_BUTTON:
            return TRUE;
    }
    return FALSE;
}

// ZNgL[ꂽƂɁAR|[lgI
static void Component_Select(Component* component, GUIApp* app)
{
    switch (component->type) {
        case COMPONENT_TEXTFIELD:
            break;

        case COMPONENT_COMBOBOX:
            if (component->component.combobox.selected) {
                component->component.combobox.index = component->component.combobox.cursor;
                component->component.combobox.selected = FALSE;
            }
            else {
                component->component.combobox.cursor = component->component.combobox.index;
                component->component.combobox.selected = TRUE;
            }
            break;

        case COMPONENT_RADIOBUTTON:
            if (!component->component.radiobutton.checked) {
                int i;

                for (i = 0; i < app->component_length; i++) {
                    if (app->components[i].type == COMPONENT_RADIOBUTTON) {
                        app->components[i].component.radiobutton.checked = FALSE;
                    }
                }
                component->component.radiobutton.checked = TRUE;
            }
            break;

        case COMPONENT_CHECKBOX:
            component->component.checkbox.checked ^= 1;
            break;

        case COMPONENT_BUTTON:
            if (app->component_index == 16) {
                DBGPRINTF("** Create!");
            }
            else if (app->component_index == 17) {
                DBGPRINTF("** Cancel!");
            }
            break;
    }
    return;
}

// R|[lgIĂꍇɃCxg
static int Component_HandleEvent(Component* component, GUIApp* app, AEEEvent evt, uint16 wp, uint32 dwp)
{
    switch (component->type) {
        case COMPONENT_TEXTFIELD:
            if (ITEXTCTL_IsActive(component->component.textctl)) {
                if (ITEXTCTL_HandleEvent(component->component.textctl, evt, wp, dwp)) {
                    return TRUE;
                }
                else if (evt == EVT_CTL_TAB) {
                    // ʂ̃R|[lg
                    ITEXTCTL_SetActive(component->component.textctl, FALSE);
                    if (wp == 0) {
                        SetFocusPreviousComponent(app);
                    }
                    else {
                        SetFocusNextComponent(app);
                    }
                    return TRUE;
                }
            }
            break;

        case COMPONENT_COMBOBOX:
            if (component->component.combobox.selected && evt == EVT_KEY) {
                switch (wp) {
                    case AVK_DOWN:
                        if (component->component.combobox.cursor < component->component.combobox.size - 1) {
                            component->component.combobox.cursor++;
                        }
                        return TRUE;

                    case AVK_UP:
                        if (component->component.combobox.cursor > 0) {
                            component->component.combobox.cursor--;
                        }
                        return TRUE;

                    case AVK_CLR:
                        component->component.combobox.selected = FALSE;
                        return TRUE;
                }
            }
            break;
    }
    return FALSE;
}
